@@ -4,10 +4,11 @@ HANDLERS = [
     ('file',         'Static content'),
     ('dirlist',      'Only listing'),
     ('redir',        'Redirection'),
-    ('cgi',          'CGI'),
     ('fcgi',         'FastCGI'),
     ('scgi',         'SCGI'),
     ('proxy',        'HTTP reverse proxy'),
+    ('streaming',    'Audio/Video streaming'),
+    ('cgi',          'CGI'),
     ('ssi',          'Server Side Includes'),
     ('server_info',  'Server Info'),
     ('dbslayer',     'MySQL bridge'),
@@ -95,6 +96,7 @@ ERROR_CODES = [
     ('403', '403 Forbidden'),
     ('404', '404 Not Found'),
     ('405', '405 Method Not Allowed'),
+    ('410', '410 Gone'),
     ('413', '413 Request Entity too large'),
     ('414', '414 Request-URI too long'),
     ('416', '416 Requested range not satisfiable'),
@@ -127,3 +129,9 @@ CRYPTORS = [
     ('',         'No TLS/SSL'),
     ('libssl',   'OpenSSL / libssl')
 ]
+
+CLIENT_CERTS = [
+    ('',         'Skip'),
+    ('accept',   'Accept'),
+    ('required', 'Require')
+]
