@@ -48,10 +48,11 @@
 
 #define ENTRIES "crypto,ssl"
 
+
 static ret_t
 _free (cherokee_cryptor_libssl_t *cryp)
 {
-	UNUSED(cryp);
+	cherokee_cryptor_free_base (CRYPTOR(cryp));
 	return ret_ok;
 }
 
@@ -131,21 +132,23 @@ openssl_sni_servername_cb (SSL *ssl, int *ad, void *arg)
 		PRINT_ERROR ("Could change the SSL context: servername='%s'\n", servername);
 	}
 
+	/* SSL_set_SSL_CTX() only change certificates. We need to
+	 * changes more options by hand.
+	 */
+	SSL_set_options(ssl, SSL_CTX_get_options(ssl->ctx));
+	
+	if ((SSL_get_verify_mode(ssl) == SSL_VERIFY_NONE) ||
+	    (SSL_num_renegotiations(ssl) == 0)) {
+
+		SSL_set_verify(ssl, SSL_CTX_get_verify_mode(ssl->ctx),
+		               SSL_CTX_get_verify_callback(ssl->ctx));
+	}
+
 	return SSL_TLSEXT_ERR_OK; 
 }
 #endif
 
 
-static int
-openssl_verify_peer (int ok, X509_STORE_CTX *x509_store)
-{
-	UNUSED(ok);
-	UNUSED(x509_store);
-
-	return 1;
-}
-
-
 static ret_t
 _vserver_new (cherokee_cryptor_t          *cryp,
 	      cherokee_virtual_server_t   *vsrv,
@@ -153,6 +156,7 @@ _vserver_new (cherokee_cryptor_t          *cryp,
 {
 	ret_t       ret;
 	int         rc;
+	int         verify_mode = SSL_VERIFY_NONE;
 	const char *error;
 	CHEROKEE_NEW_STRUCT (n, cryptor_vserver_libssl);
 
@@ -174,42 +178,26 @@ _vserver_new (cherokee_cryptor_t          *cryp,
 		return ret_error;
 	}
 
-	/* Set OpenSSL context options
+	/* Work around all known bugs
 	 */
+	SSL_CTX_set_options (n->context, SSL_OP_ALL);
 
-	/* Client-side options */
-	SSL_CTX_set_options (n->context, SSL_OP_MICROSOFT_SESS_ID_BUG);
-	SSL_CTX_set_options (n->context, SSL_OP_NETSCAPE_CHALLENGE_BUG);
-	SSL_CTX_set_options (n->context, SSL_OP_NETSCAPE_REUSE_CIPHER_CHANGE_BUG);
-
-	/* Server-side options */
-	SSL_CTX_set_options (n->context, SSL_OP_SSLREF2_REUSE_CERT_TYPE_BUG);
-	SSL_CTX_set_options (n->context, SSL_OP_MICROSOFT_BIG_SSLV3_BUFFER);
-	SSL_CTX_set_options (n->context, SSL_OP_MSIE_SSLV2_RSA_PADDING);
-	SSL_CTX_set_options (n->context, SSL_OP_SSLEAY_080_CLIENT_DH_BUG);
-	SSL_CTX_set_options (n->context, SSL_OP_TLS_D5_BUG);
-	SSL_CTX_set_options (n->context, SSL_OP_TLS_BLOCK_PADDING_BUG);
+	/* Set other OpenSSL context options
+	 */
 	SSL_CTX_set_options (n->context, SSL_OP_SINGLE_DH_USE);
 
 #ifdef SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS
 	SSL_CTX_set_options (n->context, SSL_OP_DONT_INSERT_EMPTY_FRAGMENTS);
 #endif
 
-	/* Trusted CA certificates
-	 */
-	if (! cherokee_buffer_is_empty (&vsrv->certs_ca)) {
-		rc = SSL_CTX_load_verify_locations (n->context, vsrv->certs_ca.buf, NULL);
-		if (rc != 1) {
-			OPENSSL_LAST_ERROR(error);
-			PRINT_ERROR("ERROR: OpenSSL: Can't read trusted CA list '%s': %s\n", 
-				    vsrv->server_key.buf, error);
-			return ret_error;
-		}
-	}
-
 	/* Certificate
 	 */
-	rc = SSL_CTX_use_certificate_file (n->context, vsrv->server_cert.buf, SSL_FILETYPE_PEM);
+#if (OPENSSL_VERSION_NUMBER < 0x0090808fL)
+	/* OpenSSL < 0.9.8h
+	 */
+	ERR_clear_error();
+#endif
+	rc = SSL_CTX_use_certificate_chain_file (n->context, vsrv->server_cert.buf);
 	if (rc != 1) {
 		OPENSSL_LAST_ERROR(error);
 		PRINT_ERROR("ERROR: OpenSSL: Can not use certificate file '%s':  %s\n", 
@@ -235,6 +223,45 @@ _vserver_new (cherokee_cryptor_t          *cryp,
 		return ret_error;
 	}
 
+	if (! cherokee_buffer_is_empty (&vsrv->req_client_certs)) {
+		STACK_OF(X509_NAME) *X509_clients;
+
+		verify_mode = SSL_VERIFY_PEER|SSL_VERIFY_CLIENT_ONCE;
+		if (cherokee_buffer_cmp_str (&vsrv->req_client_certs, "required") == 0) {
+			verify_mode |= SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+		}
+
+		/* Trusted CA certificates
+		*/
+		if (! cherokee_buffer_is_empty (&vsrv->certs_ca)) {
+			rc = SSL_CTX_load_verify_locations (n->context, vsrv->certs_ca.buf, NULL);
+			if (rc != 1) {
+				OPENSSL_LAST_ERROR(error);
+				PRINT_ERROR("ERROR: OpenSSL: Can't read trusted CA list '%s': %s\n", 
+				            vsrv->certs_ca.buf, error);
+				return ret_error;
+			}
+
+			X509_clients = SSL_load_client_CA_file (vsrv->certs_ca.buf);
+			if (X509_clients == NULL) {
+				OPENSSL_LAST_ERROR(error);
+				PRINT_ERROR ("SSL_load_client_CA_file '%s': %s\n",
+				             vsrv->certs_ca.buf, error);
+				return ret_error;
+			}
+
+			ERR_clear_error();
+
+			SSL_CTX_set_client_CA_list (n->context, X509_clients);
+			TRACE (ENTRIES, "Setting client CA list: %s on '%s'\n", vsrv->certs_ca.buf, vsrv->name.buf);
+		} else {
+			verify_mode = SSL_VERIFY_NONE;
+		}
+	}
+
+	SSL_CTX_set_verify (n->context, verify_mode, NULL);
+	SSL_CTX_set_verify_depth (n->context, vsrv->verify_depth);
+
 #ifndef OPENSSL_NO_TLSEXT
 	/* Enable SNI
 	 */
@@ -253,34 +280,8 @@ _vserver_new (cherokee_cryptor_t          *cryp,
 	}
 #endif /* OPENSSL_NO_TLSEXT */
 
-	if (! cherokee_buffer_is_empty (&vsrv->certs_client)) {
-		STACK_OF(X509_NAME) *X509_clients;
-
-		SSL_CTX_set_verify (n->context, SSL_VERIFY_PEER, openssl_verify_peer);
-		/* SSL_CTX_set_verify_depth (n->context, vsrv->certs_client_depth); */
-		
-		rc = SSL_CTX_load_verify_locations (n->context, vsrv->certs_client.buf, NULL);
-		if (rc != 1) {
-			OPENSSL_LAST_ERROR(error);
-			PRINT_ERROR ("Client certs checking '%s': %s\n", 
-				     vsrv->certs_client.buf, error);
-			return ret_error;
-		}
-		
-		X509_clients = SSL_load_client_CA_file (vsrv->certs_client.buf);
-		if (X509_clients == NULL) {
-			OPENSSL_LAST_ERROR(error);
-			PRINT_ERROR ("SSL_load_client_CA_file '%s': %s\n", 
-				     vsrv->certs_client.buf, error);
-			return ret_error;			
-		}
-		
-		ERR_clear_error();
-
-		SSL_CTX_set_client_CA_list (n->context, X509_clients);
-	}
-
 	*cryp_vsrv = CRYPTOR_VSRV(n);
+
 	return ret_ok;
 }
 
@@ -405,6 +406,7 @@ _socket_write (cherokee_cryptor_socket_libssl_t *cryp,
 {
 	int     re;
 	ssize_t len;
+	int     error;
 
 	len = SSL_write (cryp->session, buf, buf_len);
 	if (likely (len > 0) ) {
@@ -424,11 +426,27 @@ _socket_write (cherokee_cryptor_socket_libssl_t *cryp,
 	}
 
 	/* len < 0 */
+	error = errno;
+
 	re = SSL_get_error (cryp->session, len);
 	switch (re) {
 	case SSL_ERROR_WANT_READ:
 	case SSL_ERROR_WANT_WRITE:
 		return ret_eagain;
+
+	case SSL_ERROR_SYSCALL:
+		switch (error) {
+#ifdef ENOTCONN
+		case ENOTCONN:
+#endif
+		case EPIPE:
+		case ECONNRESET:
+			return ret_eof;
+		default:
+			PRINT_ERRNO_S (error, "SSL_write: unknown errno: ${errno}\n");
+		}
+		return ret_error;		
+		
 	case SSL_ERROR_SSL:
 		return ret_error;
 	}
