@@ -4,9 +4,8 @@
  *
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
- *      Ayose Cazorla Len <setepo@gulic.org>
  *
- * Copyright (C) 2001-2008 Alvaro Lopez Ortega
+ * Copyright (C) 2001-2009 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -19,9 +18,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
- */
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
+ */ 
 
 #include "common-internal.h"
 #include "handler_cgi.h"
@@ -291,8 +290,8 @@ cherokee_handler_cgi_configure (cherokee_config_node_t *conf, cherokee_server_t
 
 void
 cherokee_handler_cgi_add_env_pair (cherokee_handler_cgi_base_t *cgi_base,
-				   char *name,    int name_len,
-				   char *content, int content_len)
+				   const char *name,    int name_len,
+				   const char *content, int content_len)
 {
 	cherokee_handler_cgi_t *cgi = HDL_CGI(cgi_base);
 
